from secrets import token_bytes

import pytest

import pyseto
from pyseto import DecryptError, EncryptError, Key, VerifyError
from pyseto.versions.v4 import V4Local, V4Public

from .utils import load_key


class TestV4Local:
    """
    Tests for v4.local.
    """

    @pytest.mark.parametrize(
        "key, msg",
        [
            (b"", "key must be specified."),
            (token_bytes(65), "key length must be up to 64 bytes."),
        ],
    )
    def test_v4_local_new_with_invalid_arg(self, key, msg):
        with pytest.raises(ValueError) as err:
            Key.new(4, "local", key)
            pytest.fail("Key.new() should fail.")
        assert msg in str(err.value)

    def test_v4_local_decrypt_via_decode_with_wrong_key(self):
        k1 = Key.new(4, "local", b"our-secret")
        k2 = Key.new(4, "local", b"others-secret")
        token = pyseto.encode(k1, b"Hello world!")
        with pytest.raises(DecryptError) as err:
            pyseto.decode(k2, token)
            pytest.fail("pyseto.decode() should fail.")
        assert "Failed to decrypt." in str(err.value)

    def test_v4_local_encrypt_with_invalid_arg(self):
        k = Key.new(4, "local", b"our-secret")
        with pytest.raises(EncryptError) as err:
            k.encrypt(None)
            pytest.fail("pyseto.encrypt() should fail.")
        assert "Failed to encrypt." in str(err.value)

    @pytest.mark.parametrize(
        "nonce",
        [
            token_bytes(1),
            token_bytes(8),
            token_bytes(31),
            token_bytes(33),
            token_bytes(64),
        ],
    )
    def test_v4_local_encrypt_via_encode_with_wrong_nonce(self, nonce):
        k = Key.new(4, "local", b"our-secret")
        with pytest.raises(ValueError) as err:
            pyseto.encode(k, b"Hello world!", nonce=nonce)
            pytest.fail("pyseto.encode() should fail.")
        assert "nonce must be 32 bytes long." in str(err.value)

    @pytest.mark.parametrize(
        "paserk, msg",
        [
            ("xx.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version: xx."),
            ("k1.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version: k1."),
            ("k4.local.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK format."),
            ("k4.public.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK format."),
            ("k4.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK type: xxx."),
            ("k4.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type: public."),
        ],
    )
    def test_v4_local_from_paserk_with_invalid_args(self, paserk, msg):

        with pytest.raises(ValueError) as err:
            V4Local.from_paserk(paserk)
            pytest.fail("Key.from_paserk should fail.")
        assert msg in str(err.value)


class TestV4Public:
    """
    Tests for v4.public.
    """

    def test_v4_public_to_paserk_id(self):
        sk = Key.new(4, "public", load_key("keys/private_key_ed25519.pem"))
        pk = Key.new(4, "public", load_key("keys/public_key_ed25519.pem"))
        assert sk.to_peer_paserk_id() == pk.to_paserk_id()
        assert pk.to_peer_paserk_id() == ""

    def test_v4_public_verify_via_encode_with_wrong_key(self):
        sk = Key.new(4, "public", load_key("keys/private_key_ed25519.pem"))
        pk = Key.new(4, "public", load_key("keys/public_key_ed25519_2.pem"))
        token = pyseto.encode(sk, b"Hello world!")
        with pytest.raises(VerifyError) as err:
            pyseto.decode(pk, token)
            pytest.fail("pyseto.decode() should fail.")
        assert "Failed to verify." in str(err.value)

    @pytest.mark.parametrize(
        "paserk, msg",
        [
            ("xx.public.AAAAAAAAAAAAAAAA", "Invalid PASERK version: xx."),
            ("k1.public.AAAAAAAAAAAAAAAA", "Invalid PASERK version: k1."),
            ("k4.public.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK format."),
            ("k4.local.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK format."),
            ("k4.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK type: xxx."),
            ("k4.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type: local."),
        ],
    )
    def test_v4_public_from_paserk_with_invalid_args(self, paserk, msg):

        with pytest.raises(ValueError) as err:
            V4Public.from_paserk(paserk)
            pytest.fail("Key.from_paserk should fail.")
        assert msg in str(err.value)



import hashlib
import hmac
from secrets import token_bytes
from typing import Any, Union

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric.ec import (
    EllipticCurvePrivateKey,
    EllipticCurvePublicKey,
)
from cryptography.hazmat.primitives.asymmetric.utils import (
    decode_dss_signature,
    encode_dss_signature,
)
from cryptography.hazmat.primitives.kdf.hkdf import HKDF

from ..exceptions import DecryptError, SignError, VerifyError
from ..key_interface import KeyInterface
from ..key_nist import NISTKey
from ..utils import (
    base64url_decode,
    base64url_encode,
    ec_public_key_compress,
    i2osp,
    os2ip,
    pae,
)


class V3Local(NISTKey):
    """
    The key object for v3.local.
    """

    _VERSION = 3
    _TYPE = "local"

    def __init__(self, key: Union[str, bytes]):

        super().__init__(key)
        return

    def encrypt(
        self,
        payload: bytes,
        footer: bytes = b"",
        implicit_assertion: bytes = b"",
        nonce: bytes = b"",
    ) -> bytes:

        if nonce:
            if len(nonce) != 32:
                raise ValueError("nonce must be 32 bytes long.")
        else:
            nonce = token_bytes(32)
        e = HKDF(
            algorithm=hashes.SHA384(),
            length=48,
            salt=None,
            info=b"paseto-encryption-key" + nonce,
        )
        a = HKDF(
            algorithm=hashes.SHA384(),
            length=48,
            salt=None,
            info=b"paseto-auth-key-for-aead" + nonce,
        )
        try:
            tmp = e.derive(self._key)
            ek = tmp[0:32]
            n2 = tmp[32:]
            ak = a.derive(self._key)
        except Exception as err:
            raise DecryptError("Failed to derive keys.") from err

        c = self._encrypt(ek, n2, payload)
        pre_auth = pae([self.header, nonce, c, footer, implicit_assertion])
        t = hmac.new(ak, pre_auth, hashlib.sha384).digest()
        token = self._header + base64url_encode(nonce + c + t)
        if footer:
            token += b"." + base64url_encode(footer)
        return token

    def decrypt(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b"") -> bytes:

        n = payload[0:32]
        c = payload[32 : len(payload) - 48]
        t = payload[-48:]
        e = HKDF(
            algorithm=hashes.SHA384(),
            length=48,
            salt=None,
            info=b"paseto-encryption-key" + n,
        )
        a = HKDF(
            algorithm=hashes.SHA384(),
            length=48,
            salt=None,
            info=b"paseto-auth-key-for-aead" + n,
        )
        try:
            tmp = e.derive(self._key)
            ek = tmp[0:32]
            n2 = tmp[32:]
            ak = a.derive(self._key)
        except Exception as err:
            raise DecryptError("Failed to derive keys.") from err

        pre_auth = pae([self.header, n, c, footer, implicit_assertion])
        t2 = hmac.new(ak, pre_auth, hashlib.sha384).digest()
        if t != t2:
            raise DecryptError("Failed to decrypt.")
        return self._decrypt(ek, n2, c)

    def to_paserk_id(self) -> str:

        h = "k3.lid."
        p = self.to_paserk()
        digest = hashes.Hash(hashes.SHA384())
        digest.update((h + p).encode("utf-8"))
        d = digest.finalize()
        return h + base64url_encode(d[0:33]).decode("utf-8")


class V3Public(NISTKey):
    """
    The key object for v3.public.
    """

    _VERSION = 3
    _TYPE = "public"

    def __init__(self, key: Any):

        super().__init__(key)

        self._sig_size = 96
        if not isinstance(self._key, (EllipticCurvePublicKey, EllipticCurvePrivateKey)):
            raise ValueError("The key is not ECDSA key.")

        if isinstance(self._key, EllipticCurvePublicKey):
            self._is_secret = False
        return

    @classmethod
    def from_public_bytes(cls, key: bytes):
        try:
            k = EllipticCurvePublicKey.from_encoded_point(ec.SECP384R1(), key)
        except Exception as err:
            raise ValueError("Invalid bytes for the key.") from err
        return cls(k)

    @classmethod
    def from_paserk(
        cls,
        paserk: str,
        wrapping_key: bytes = b"",
        password: bytes = b"",
        unsealing_key: bytes = b"",
    ) -> KeyInterface:

        if wrapping_key and password:
            raise ValueError("Only one of wrapping_key or password should be specified.")

        frags = paserk.split(".")
        if frags[0] != "k3":
            raise ValueError(f"Invalid PASERK version: {frags[0]}.")

        if wrapping_key:
            # secret-wrap
            if len(frags) != 4:
                raise ValueError("Invalid PASERK format.")
            if frags[2] != "pie":
                raise ValueError(f"Unknown wrapping algorithm: {frags[2]}.")

            if frags[1] == "secret-wrap":
                h = "k3.secret-wrap.pie."
                k = cls._decode_pie(h, wrapping_key, frags[3])
                priv = ec.derive_private_key(int.from_bytes(k, byteorder="big"), ec.SECP384R1())
                return cls(priv)
            raise ValueError(f"Invalid PASERK type: {frags[1]}.")

        if len(frags) != 3:
            raise ValueError("Invalid PASERK format.")

        if password:
            # secret-pw
            if frags[1] == "secret-pw":
                h = "k3.secret-pw."
                k = cls._decode_pbkw(h, password, frags[2])
                priv = ec.derive_private_key(int.from_bytes(k, byteorder="big"), ec.SECP384R1())
                return cls(priv)
            raise ValueError(f"Invalid PASERK type: {frags[1]}.")

        # public
        k = base64url_decode(frags[2])
        if frags[1] == "public":
            pub = EllipticCurvePublicKey.from_encoded_point(ec.SECP384R1(), k)
            return cls(pub)

        # secret
        if frags[1] == "secret":
            priv = ec.derive_private_key(int.from_bytes(k, byteorder="big"), ec.SECP384R1())
            return cls(priv)
        if frags[1] == "secret-wrap":
            raise ValueError(f"{frags[1]} needs wrapping_key.")
        raise ValueError(f"Invalid PASERK type: {frags[1]}.")

    def to_paserk(
        self,
        wrapping_key: Union[bytes, str] = b"",
        password: Union[bytes, str] = b"",
        sealing_key: Union[bytes, str] = b"",
        iteration: int = 100000,
        memory_cost: int = 15 * 1024,
        time_cost: int = 2,
        parallelism: int = 1,
    ) -> str:

        if wrapping_key and password:
            raise ValueError("Only one of wrapping_key or password should be specified.")

        if wrapping_key:
            # secret-wrap
            if not isinstance(self._key, EllipticCurvePrivateKey):
                raise ValueError("Public key cannot be wrapped.")

            bkey = wrapping_key if isinstance(wrapping_key, bytes) else wrapping_key.encode("utf-8")
            h = "k3.secret-wrap.pie."
            k = self._key.private_numbers().private_value.to_bytes(48, byteorder="big")
            return h + self._encode_pie(h, bkey, k)

        if password:
            # secret-pw
            if not isinstance(self._key, EllipticCurvePrivateKey):
                raise ValueError("Public key cannot be wrapped.")

            bpw = password if isinstance(password, bytes) else password.encode("utf-8")
            h = "k3.secret-pw."
            k = self._key.private_numbers().private_value.to_bytes(48, byteorder="big")
            return h + self._encode_pbkw(h, bpw, k, iteration)

        # public
        if isinstance(self._key, EllipticCurvePublicKey):
            k = ec_public_key_compress(self._key.public_numbers().x, self._key.public_numbers().y)
            return "k3.public." + base64url_encode(k).decode("utf-8")

        # private
        k = self._key.private_numbers().private_value.to_bytes(48, byteorder="big")
        return "k3.secret." + base64url_encode(k).decode("utf-8")

    def to_paserk_id(self) -> str:
        p = self.to_paserk()
        h = "k3.pid." if isinstance(self._key, EllipticCurvePublicKey) else "k3.sid."
        digest = hashes.Hash(hashes.SHA384())
        digest.update((h + p).encode("utf-8"))
        d = digest.finalize()
        return h + base64url_encode(d[0:33]).decode("utf-8")

    def sign(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b"") -> bytes:

        if isinstance(self._key, EllipticCurvePublicKey):
            raise ValueError("A public key cannot be used for signing.")
        pk = ec_public_key_compress(
            self._key.private_numbers().public_numbers.x,
            self._key.private_numbers().public_numbers.y,
        )
        m2 = pae([pk, self.header, payload, footer, implicit_assertion])
        try:
            sig = self._key.sign(m2, ec.ECDSA(hashes.SHA384()))
            return self._der_to_os(self._key.curve.key_size, sig)
        except Exception as err:
            raise SignError("Failed to sign.") from err

    def verify(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b""):

        if len(payload) <= self._sig_size:
            raise ValueError("Invalid payload.")

        sig = payload[-self._sig_size :]
        m = payload[: len(payload) - self._sig_size]
        k = self._key if isinstance(self._key, EllipticCurvePublicKey) else self._key.public_key()
        pk = ec_public_key_compress(k.public_numbers().x, k.public_numbers().y)
        m2 = pae([pk, self.header, m, footer, implicit_assertion])
        try:
            der_sig = self._os_to_der(self._key.curve.key_size, sig)
            k.verify(der_sig, m2, ec.ECDSA(hashes.SHA384()))
        except Exception as err:
            raise VerifyError("Failed to verify.") from err
        return m

    def _der_to_os(self, key_size: int, sig: bytes) -> bytes:

        num_bytes = (key_size + 7) // 8
        r, s = decode_dss_signature(sig)
        return i2osp(r, num_bytes) + i2osp(s, num_bytes)

    def _os_to_der(self, key_size: int, sig: bytes) -> bytes:

        num_bytes = (key_size + 7) // 8
        if len(sig) != 2 * num_bytes:
            raise ValueError("Invalid signature.")
        r = os2ip(sig[:num_bytes])
        s = os2ip(sig[num_bytes:])
        return encode_dss_signature(r, s)



import hashlib
import hmac
from secrets import token_bytes
from typing import Any, Union

from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives.serialization import (
    load_der_private_key,
    load_der_public_key,
)

from ..exceptions import DecryptError, SignError, VerifyError
from ..key_interface import KeyInterface
from ..key_nist import NISTKey
from ..utils import base64url_decode, base64url_encode, pae


class V1Local(NISTKey):
    """
    The key object for v1.local.
    """

    _VERSION = 1
    _TYPE = "local"

    def __init__(self, key: Union[str, bytes]):

        super().__init__(key)
        return

    def to_paserk_id(self) -> str:
        h = "k1.lid."
        p = self.to_paserk()
        digest = hashes.Hash(hashes.SHA384())
        digest.update((h + p).encode("utf-8"))
        d = digest.finalize()
        return h + base64url_encode(d[0:33]).decode("utf-8")

    def encrypt(
        self,
        payload: bytes,
        footer: bytes = b"",
        implicit_assertion: bytes = b"",
        nonce: bytes = b"",
    ) -> bytes:

        if nonce:
            if len(nonce) != 32:
                raise ValueError("nonce must be 32 bytes long.")
        else:
            nonce = token_bytes(32)

        n = self._generate_hash(nonce, payload, 32)
        e = HKDF(
            algorithm=hashes.SHA384(),
            length=32,
            salt=n[0:16],
            info=b"paseto-encryption-key",
        )
        a = HKDF(
            algorithm=hashes.SHA384(),
            length=32,
            salt=n[0:16],
            info=b"paseto-auth-key-for-aead",
        )
        ek = e.derive(self._key)
        ak = a.derive(self._key)

        c = self._encrypt(ek, n[16:], payload)
        pre_auth = pae([self.header, n, c, footer])
        t = hmac.new(ak, pre_auth, hashlib.sha384).digest()
        token = self._header + base64url_encode(n + c + t)
        if footer:
            token += b"." + base64url_encode(footer)
        return token

    def decrypt(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b"") -> bytes:

        n = payload[0:32]
        t = payload[-48:]
        c = payload[32 : len(payload) - 48]
        e = HKDF(
            algorithm=hashes.SHA384(),
            length=32,
            salt=n[0:16],
            info=b"paseto-encryption-key",
        )
        a = HKDF(
            algorithm=hashes.SHA384(),
            length=32,
            salt=n[0:16],
            info=b"paseto-auth-key-for-aead",
        )
        ek = e.derive(self._key)
        ak = a.derive(self._key)

        pre_auth = pae([self.header, n, c, footer])
        t2 = hmac.new(ak, pre_auth, hashlib.sha384).digest()
        if t != t2:
            raise DecryptError("Failed to decrypt.")
        return self._decrypt(ek, n[16:], c)


class V1Public(NISTKey):
    """
    The key object for v1.public.
    """

    _VERSION = 1
    _TYPE = "public"

    def __init__(self, key: Any):

        super().__init__(key)

        self._sig_size = 256
        if not isinstance(self._key, (RSAPublicKey, RSAPrivateKey)):
            raise ValueError("The key is not RSA key.")

        if isinstance(self._key, RSAPublicKey):
            self._is_secret = False

        self._padding = padding.PSS(mgf=padding.MGF1(hashes.SHA384()), salt_length=48)
        return

    @classmethod
    def from_paserk(
        cls,
        paserk: str,
        wrapping_key: bytes = b"",
        password: bytes = b"",
        unsealing_key: bytes = b"",
    ) -> KeyInterface:

        if wrapping_key and password:
            raise ValueError("Only one of wrapping_key or password should be specified.")

        frags = paserk.split(".")
        if frags[0] != "k1":
            raise ValueError(f"Invalid PASERK version: {frags[0]}.")

        if wrapping_key:
            if len(frags) != 4:
                raise ValueError("Invalid PASERK format.")
            if frags[2] != "pie":
                raise ValueError(f"Unknown wrapping algorithm: {frags[2]}.")

            if frags[1] == "secret-wrap":
                h = "k1.secret-wrap.pie."
                k = cls._decode_pie(h, wrapping_key, frags[3])
                return cls(load_der_private_key(k, password=None))
            raise ValueError(f"Invalid PASERK type: {frags[1]}.")

        if len(frags) != 3:
            raise ValueError("Invalid PASERK format.")

        if password:
            if frags[1] == "secret-pw":
                h = "k1.secret-pw."
                k = cls._decode_pbkw(h, password, frags[2])
                return cls(load_der_private_key(k, password=None))
            raise ValueError(f"Invalid PASERK type: {frags[1]}.")

        wrapped = base64url_decode(frags[2])
        if frags[1] == "public":
            return cls(load_der_public_key(wrapped))
        if frags[1] == "secret":
            return cls(load_der_private_key(wrapped, password=None))
        if frags[1] == "secret-wrap":
            raise ValueError(f"{frags[1]} needs wrapping_key.")
        raise ValueError(f"Invalid PASERK type: {frags[1]}.")

    def to_paserk(
        self,
        wrapping_key: Union[bytes, str] = b"",
        password: Union[bytes, str] = b"",
        sealing_key: Union[bytes, str] = b"",
        iteration: int = 100000,
        memory_cost: int = 15 * 1024,
        time_cost: int = 2,
        parallelism: int = 1,
    ) -> str:

        if wrapping_key and password:
            raise ValueError("Only one of wrapping_key or password should be specified.")

        if wrapping_key:
            # secret-wrap
            if not isinstance(self._key, RSAPrivateKey):
                raise ValueError("Public key cannot be wrapped.")

            bkey = wrapping_key if isinstance(wrapping_key, bytes) else wrapping_key.encode("utf-8")
            h = "k1.secret-wrap.pie."
            priv = self._key.private_bytes(
                encoding=serialization.Encoding.DER,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption(),
            )
            return h + self._encode_pie(h, bkey, priv)

        if password:
            # secret-pw
            if not isinstance(self._key, RSAPrivateKey):
                raise ValueError("Public key cannot be wrapped.")

            bpw = password if isinstance(password, bytes) else password.encode("utf-8")
            h = "k1.secret-pw."
            priv = self._key.private_bytes(
                encoding=serialization.Encoding.DER,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption(),
            )
            return h + self._encode_pbkw(h, bpw, priv, iteration)

        # public
        if isinstance(self._key, RSAPublicKey):
            pub = self._key.public_bytes(
                encoding=serialization.Encoding.DER,
                format=serialization.PublicFormat.SubjectPublicKeyInfo,
            )
            return "k1.public." + base64url_encode(pub).decode("utf-8")

        # secret
        priv = self._key.private_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption(),
        )
        return "k1.secret." + base64url_encode(priv).decode("utf-8")

    def to_paserk_id(self) -> str:
        p = self.to_paserk()
        h = "k1.pid." if isinstance(self._key, RSAPublicKey) else "k1.sid."
        digest = hashes.Hash(hashes.SHA384())
        digest.update((h + p).encode("utf-8"))
        d = digest.finalize()
        return h + base64url_encode(d[0:33]).decode("utf-8")

    def sign(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b"") -> bytes:

        if isinstance(self._key, RSAPublicKey):
            raise ValueError("A public key cannot be used for signing.")
        m2 = pae([self.header, payload, footer])
        try:
            return self._key.sign(m2, self._padding, hashes.SHA384())
        except Exception as err:
            raise SignError("Failed to sign.") from err

    def verify(self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b""):

        if len(payload) <= self._sig_size:
            raise ValueError("Invalid payload.")

        sig = payload[-self._sig_size :]
        m = payload[: len(payload) - self._sig_size]
        k = self._key if isinstance(self._key, RSAPublicKey) else self._key.public_key()
        m2 = pae([self.header, m, footer])
        try:
            k.verify(sig, m2, self._padding, hashes.SHA384())
        except Exception as err:
            raise VerifyError("Failed to verify.") from err
        return m



import pytest

from pyseto.token import Token
from pyseto.utils import base64url_decode


class TestToken:
    """
    Tests for Token.
    """

    def test_token_new(self):
        token = Token.new(
            b"v1.local.WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        assert token.version == "v1"
        assert token.purpose == "local"
        assert token.payload == base64url_decode(
            "WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        assert token.footer == b""

    def test_token_new_with_str(self):
        token = Token.new(
            "v1.local.WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        assert token.version == "v1"
        assert token.purpose == "local"
        assert token.payload == base64url_decode(
            "WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        assert token.footer == b""

    def test_token_setter_payload(self):
        token = Token.new(
            b"v1.local.WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        token.payload = b"updated-payload"
        assert token.payload == b"updated-payload"

    def test_token_setter_footer(self):
        token = Token.new(
            b"v1.local.WzhIh1MpbqVNXNt7-HbWvL-JwAym3Tomad9Pc2nl7wK87vGraUVvn2bs8BBNo7jbukCNrkVID0jCK2vr5bP18G78j1bOTbBcP9HZzqnraEdspcjd_PvrxDEhj9cS2MG5fmxtvuoHRp3M24HvxTtql9z26KTfPWxJN5bAJaAM6gos8fnfjJO8oKiqQMaiBP_Cqncmqw8"
        )
        token.footer = b"updated-footer"
        assert token.footer == b"updated-footer"

    @pytest.mark.parametrize(
        "token, msg",
        [
            ("v1", "token is invalid."),
            ("v1.", "token is invalid."),
            ("v1.local", "token is invalid."),
            ("v1.local.", "Empty payload."),
            ("v1.local.p.f.x", "token is invalid."),
            ("v1.local.p.f.x.y", "token is invalid."),
        ],
    )
    def test_token_new_with_invalid_token(self, token, msg):
        with pytest.raises(ValueError) as err:
            Token.new(token)
            pytest.fail("Token.new() should fail.")
        assert msg in str(err.value)



from secrets import token_bytes

import pytest

from pyseto import DecryptError, Key, NotSupportedError
from pyseto.key_interface import KeyInterface
from pyseto.utils import base64url_decode

from .utils import load_jwk, load_key


class TestKey:
    """
    Tests for Key.
    """

    @pytest.mark.parametrize(
        "version, purpose, key",
        [
            (1, "local", token_bytes(32)),
            (2, "local", token_bytes(32)),
            (3, "local", token_bytes(32)),
            (4, "local", token_bytes(32)),
        ],
    )
    def test_key_new_local(self, version, purpose, key):
        k = Key.new(version, purpose, key)
        assert isinstance(k, KeyInterface)
        assert k.version == version
        assert k.purpose == purpose
        with pytest.raises(NotSupportedError) as err:
            k.sign(b"Hello world!")
            pytest.fail("Key.sign() should fail.")
        assert "A key for local does not have sign()." in str(err.value)
        with pytest.raises(NotSupportedError) as err:
            k.verify(b"xxxxxx")
            pytest.fail("Key.verify() should fail.")
        assert "A key for local does not have verify()." in str(err.value)

    @pytest.mark.parametrize(
        "version, purpose, key",
        [
            (1, "public", load_key("keys/private_key_rsa.pem")),
            (1, "public", load_key("keys/public_key_rsa.pem")),
            (2, "public", load_key("keys/private_key_ed25519.pem")),
            (2, "public", load_key("keys/public_key_ed25519.pem")),
            (3, "public", load_key("keys/private_key_ecdsa_p384.pem")),
            (3, "public", load_key("keys/public_key_ecdsa_p384.pem")),
            (4, "public", load_key("keys/private_key_ed25519.pem")),
            (4, "public", load_key("keys/public_key_ed25519.pem")),
        ],
    )
    def test_key_new_public(self, version, purpose, key):
        k = Key.new(version, purpose, key)
        assert isinstance(k, KeyInterface)
        assert k.version == version
        assert k.purpose == purpose
        with pytest.raises(NotSupportedError) as err:
            k.encrypt(b"Hello world!")
            pytest.fail("Key.sign() should fail.")
        assert "A key for public does not have encrypt()." in str(err.value)
        with pytest.raises(NotSupportedError) as err:
            k.decrypt(b"xxxxxx")
            pytest.fail("Key.verify() should fail.")
        assert "A key for public does not have decrypt()." in str(err.value)

    @pytest.mark.parametrize(
        "version, key, msg",
        [
            (1, load_key("keys/private_key_ed25519.pem"), "The key is not RSA key."),
            (1, load_key("keys/public_key_ed25519.pem"), "The key is not RSA key."),
            (
                1,
                load_key("keys/private_key_ecdsa_p384.pem"),
                "The key is not RSA key.",
            ),
            (
                1,
                load_key("keys/public_key_ecdsa_p384.pem"),
                "The key is not RSA key.",
            ),
            (2, load_key("keys/private_key_rsa.pem"), "The key is not Ed25519 key."),
            (2, load_key("keys/public_key_rsa.pem"), "The key is not Ed25519 key."),
            (
                2,
                load_key("keys/private_key_ecdsa_p384.pem"),
                "The key is not Ed25519 key.",
            ),
            (
                2,
                load_key("keys/public_key_ecdsa_p384.pem"),
                "The key is not Ed25519 key.",
            ),
            (
                3,
                load_key("keys/private_key_ed25519.pem"),
                "The key is not ECDSA key.",
            ),
            (
                3,
                load_key("keys/public_key_ed25519.pem"),
                "The key is not ECDSA key.",
            ),
            (3, load_key("keys/private_key_rsa.pem"), "The key is not ECDSA key."),
            (3, load_key("keys/public_key_rsa.pem"), "The key is not ECDSA key."),
            (4, load_key("keys/private_key_rsa.pem"), "The key is not Ed25519 key."),
            (4, load_key("keys/public_key_rsa.pem"), "The key is not Ed25519 key."),
            (
                4,
                load_key("keys/private_key_ecdsa_p384.pem"),
                "The key is not Ed25519 key.",
            ),
            (
                4,
                load_key("keys/public_key_ecdsa_p384.pem"),
                "The key is not Ed25519 key.",
            ),
        ],
    )
    def test_key_new_public_with_wrong_key(self, version, key, msg):
        with pytest.raises(ValueError) as err:
            Key.new(version, "public", key)
            pytest.fail("Key.new should fail.")
        assert msg in str(err.value)

    @pytest.mark.parametrize(
        "version, purpose, key, msg",
        [
            ("v*", "local", token_bytes(32), "Invalid version: v*."),
            ("v0", "local", token_bytes(32), "Invalid version: v0."),
            (0, "local", token_bytes(32), "Invalid version: 0."),
            (
                "v*",
                "public",
                load_key("keys/private_key_rsa.pem"),
                "Invalid version: v*.",
            ),
            (
                "v0",
                "public",
                load_key("keys/private_key_rsa.pem"),
                "Invalid version: v0.",
            ),
            (
                0,
                "public",
                load_key("keys/private_key_rsa.pem"),
                "Invalid version: 0.",
            ),
            (1, "xxx", token_bytes(32), "Invalid purpose: xxx."),
            (1, "public", "-----BEGIN BAD", "Invalid or unsupported PEM format."),
        ],
    )
    def test_key_new_with_invalid_arg(self, version, purpose, key, msg):
        with pytest.raises(ValueError) as err:
            Key.new(version, purpose, key)
            pytest.fail("Key.new should fail.")
        assert msg in str(err.value)

    @pytest.mark.parametrize(
        "version, key",
        [
            # (1, load_jwk("keys/private_key_rsa.json")),
            # (1, load_jwk("keys/public_key_rsa.json")),
            (2, load_jwk("keys/private_key_ed25519.json")),
            (2, load_jwk("keys/public_key_ed25519.json")),
            (3, load_jwk("keys/private_key_ecdsa_p384.json")),
            (3, load_jwk("keys/public_key_ecdsa_p384.json")),
            (4, load_jwk("keys/private_key_ed25519.json")),
            (4, load_jwk("keys/public_key_ed25519.json")),
        ],
    )
    def test_key_from_asymmetric_params(self, version, key):

        k = Key.from_asymmetric_key_params(version, x=key["x"], y=key["y"], d=key["d"])
        assert isinstance(k, KeyInterface)
        assert k.version == version
        assert k.purpose == "public"

    @pytest.mark.parametrize(
        "paserk",
        [
            "k1.local.AAAAAAAAAAAAAAAA",
            "k1.public.AAAAAAAAAAAAAAAA",
            "k2.local.AAAAAAAAAAAAAAAA",
            "k2.public.AAAAAAAAAAAAAAAA",
            "k3.local.AAAAAAAAAAAAAAAA",
            "k3.public.AAAAAAAAAAAAAAAA",
            "k4.local.AAAAAAAAAAAAAAAA",
            "k4.public.AAAAAAAAAAAAAAAA",
        ],
    )
    def test_key_from_paserk_with_wrapping_key_and_password(self, paserk):

        with pytest.raises(ValueError) as err:
            Key.from_paserk(paserk, wrapping_key="xxx", password="yyy")
            pytest.fail("Key.from_paserk should fail.")
        assert "Only one of wrapping_key or password should be specified." in str(err.value)

    @pytest.mark.parametrize(
        "paserk, msg",
        [
            ("k1.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type: local."),
            ("k1.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type: public."),
            ("k2.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type: local."),
            ("k2.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type: public."),
            ("k3.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type: local."),
            ("k3.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type: public."),
            ("k4.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type: local."),
            ("k4.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type: public."),
        ],
    )
    def test_key_from_paserk_with_password_for_wrong_paserk(self, paserk, msg):

        with pytest.raises(ValueError) as err:
            Key.from_paserk(paserk, password="yyy")
            pytest.fail("Key.from_paserk should fail.")
        assert msg in str(err.value)

    @pytest.mark.parametrize(
        "paserk, msg",
        [
            ("v1.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version: v1."),
            ("*.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version: *."),
            ("k1.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK key type: xxx."),
        ],
    )
    def test_key_from_paserk_with_invalid_args(self, paserk, msg):

        with pytest.raises(ValueError) as err:
            Key.from_paserk(paserk)
            pytest.fail("Key.from_paserk should fail.")
        assert msg in str(err.value)

    @pytest.mark.parametrize(
        "version",
        [
            1,
            2,
            3,
            4,
        ],
    )
    def test_key_from_paserk_for_local_with_wrong_wrapping_key(self, version):
        k = Key.new(version, "local", token_bytes(32))
        wk1 = token_bytes(32)
        wk2 = token_bytes(32)
        wpk = k.to_paserk(wrapping_key=wk1)
        with pytest.raises(DecryptError) as err:
            Key.from_paserk(wpk, wrapping_key=wk2)
            pytest.fail("Key.from_paserk() should fail.")
        assert "Failed to unwrap a key." in str(err.value)

    @pytest.mark.parametrize(
        "version",
        [
            1,
            2,
            3,
            4,
        ],
    )
    def test_key_from_paserk_for_local_with_wrong_password(self, version):
        k = Key.new(version, "local", token_bytes(32))
        wk1 = token_bytes(32)
        wk2 = token_bytes(32)
        wpk = k.to_paserk(password=wk1)
        with pytest.raises(DecryptError) as err:
            Key.from_paserk(wpk, password=wk2)
            pytest.fail("Key.from_paserk() should fail.")
        assert "Failed to unwrap a key." in str(err.value)

    @pytest.mark.parametrize(
        "version, key",
        [
            (1, load_key("keys/private_key_rsa.pem")),
            (2, load_key("keys/private_key_ed25519.pem")),
            (3, load_key("keys/private_key_ecdsa_p384.pem")),
            (4, load_key("keys/private_key_ed25519.pem")),
        ],
    )
    def test_key_from_paserk_for_private_key_with_wrong_wrapping_key(self, version, key):
        k = Key.new(version, "public", key)
        wk1 = token_bytes(32)
        wk2 = token_bytes(32)
        wpk = k.to_paserk(wrapping_key=wk1)
        with pytest.raises(DecryptError) as err:
            Key.from_paserk(wpk, wrapping_key=wk2)
            pytest.fail("Key.from_paserk() should fail.")
        assert "Failed to unwrap a key." in str(err.value)

    @pytest.mark.parametrize(
        "version, key",
        [
            (1, load_key("keys/public_key_rsa.pem")),
            (2, load_key("keys/public_key_ed25519.pem")),
            (3, load_key("keys/public_key_ecdsa_p384.pem")),
            (4, load_key("keys/public_key_ed25519.pem")),
        ],
    )
    def test_key_from_paserk_for_public_key_with_wrapping_key(self, version, key):
        k = Key.new(version, "public", key)
        wk = token_bytes(32)
        with pytest.raises(ValueError) as err:
            k.to_paserk(wrapping_key=wk)
            pytest.fail("to_paserk() should fail.")
        assert "Public key cannot be wrapped." in str(err.value)

    @pytest.mark.parametrize(
        "version, key",
        [
            (1, load_key("keys/public_key_rsa.pem")),
            (2, load_key("keys/public_key_ed25519.pem")),
            (3, load_key("keys/public_key_ecdsa_p384.pem")),
            (4, load_key("keys/public_key_ed25519.pem")),
        ],
    )
    def test_key_from_paserk_for_public_key_with_password(self, version, key):
        k = Key.new(version, "public", key)
        wk = token_bytes(32)
        with pytest.raises(ValueError) as err:
            k.to_paserk(password=wk)
            pytest.fail("to_paserk() should fail.")
        assert "Public key cannot be wrapped." in str(err.value)

    @pytest.mark.parametrize(
        "version, key, msg",
        [
            (
                1,
                load_jwk("keys/private_key_rsa.json"),
                "v1.public is not supported on from_key_parameters.",
            ),
            (999, load_jwk("keys/private_key_ed25519.json"), "Invalid version: 999."),
            (0, load_jwk("keys/private_key_ed25519.json"), "Invalid version: 0."),
            (
                2,
                {"x": b"xxx", "y": b"", "d": b"ddd"},
                "Only one of x or d should be set for v2.public.",
            ),
            (2, {"x": b"xxx", "y": b"", "d": b""}, "Failed to load key."),
            (2, {"x": b"", "y": b"", "d": b"ddd"}, "Failed to load key."),
            (
                2,
                {"x": b"", "y": b"", "d": b""},
                "x or d should be set for v2.public.",
            ),
            (
                3,
                {"x": b"xxx", "y": b"", "d": b"ddd"},
                "x and y (and d) should be set for v3.public.",
            ),
            (
                3,
                {"x": b"", "y": b"yyy", "d": b"ddd"},
                "x and y (and d) should be set for v3.public.",
            ),
            (3, {"x": b"xxx", "y": b"yyy", "d": b""}, "Failed to load key."),
            (
                3,
                {
                    "x": base64url_decode("_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI"),
                    "y": base64url_decode("GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50"),
                    "d": b"ddd",
                },
                "Failed to load key.",
            ),
            (
                4,
                {"x": b"xxx", "y": b"", "d": b"ddd"},
                "Only one of x or d should be set for v4.public.",
            ),
            (4, {"x": b"xxx", "y": b"", "d": b""}, "Failed to load key."),
            (4, {"x": b"", "y": b"", "d": b"ddd"}, "Failed to load key."),
            (
                4,
                {"x": b"", "y": b"", "d": b""},
                "x or d should be set for v4.public.",
            ),
        ],
    )
    def test_key_from_asymmetric_params_with_invalid_arg(self, version, key, msg):

        with pytest.raises(ValueError) as err:
            Key.from_asymmetric_key_params(version, x=key["x"], y=key["y"], d=key["d"])
            pytest.fail("Key.from_asymmetric_key_params() should fail.")
        assert msg in str(err.value)

    @pytest.mark.parametrize(
        "version, purpose, key",
        [
            (1, "public", load_key("keys/public_key_rsa.pem")),
            (2, "public", load_key("keys/public_key_ed25519.pem")),
            (3, "public", load_key("keys/public_key_ecdsa_p384.pem")),
            (4, "public", load_key("keys/public_key_ed25519.pem")),
        ],
    )
    def test_key_to_paserk_public(self, version, purpose, key):
        k = Key.new(version, purpose, key)
        assert k.to_paserk().startswith(f"k{k.version}.public.")

    @pytest.mark.parametrize(
        "version, purpose, key",
        [
            (1, "public", load_key("keys/private_key_rsa.pem")),
            (2, "public", load_key("keys/private_key_ed25519.pem")),
            (3, "public", load_key("keys/private_key_ecdsa_p384.pem")),
            (4, "public", load_key("keys/private_key_ed25519.pem")),
        ],
    )
    def test_key_to_paserk_secret(self, version, purpose, key):
        k = Key.new(version, purpose, key)
        assert k.to_paserk().startswith(f"k{k.version}.secret.")

    @pytest.mark.parametrize(
        "version, purpose, key",
        [
            (1, "local", token_bytes(32)),
            (2, "local", token_bytes(32)),
            (3, "local", token_bytes(32)),
            (4, "local", token_bytes(32)),
            (1, "public", load_key("keys/private_key_rsa.pem")),
            (2, "public", load_key("keys/private_key_ed25519.pem")),
            (3, "public", load_key("keys/private_key_ecdsa_p384.pem")),
            (4, "public", load_key("keys/private_key_ed25519.pem")),
        ],
    )
    def test_key_to_paserk_secret_with_wrapping_key_and_password(self, version, purpose, key):
        k = Key.new(version, purpose, key)
        with pytest.raises(ValueError) as err:
            k.to_paserk(wrapping_key="xxx", password="yyy")
            pytest.fail("to_paserk() should fail.")
        assert "Only one of wrapping_key or password should be specified." in str(err.value)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyseto', 'pyseto.versions']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=36.0.0,<37.0.0',
 'iso8601>=1.0.2,<2.0.0',
 'passlib[argon2]>=1.7.4,<2.0.0',
 'pycryptodomex>=3.12.0,<4.0.0']

extras_require = \
{':extra == "docs"': ['Sphinx[docs]>=4.3.1,<5.0.0',
                      'sphinx-autodoc-typehints[docs]>=1.12.0,<2.0.0',
                      'sphinx-rtd-theme[docs]>=1.0.0,<2.0.0']}

setup_kwargs = {
    'name': 'pyseto',
    'version': '1.6.2',
    'description': 'A Python implementation of PASETO/PASERK.',
    'long_description': '# PySETO - A Python implementation of PASETO/PASERK\n\n[![PyPI version](https://badge.fury.io/py/pyseto.svg)](https://badge.fury.io/py/pyseto)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pyseto)\n[![Documentation Status](https://readthedocs.org/projects/pyseto/badge/?version=latest)](https://pyseto.readthedocs.io/en/latest/?badge=latest)\n![Github CI](https://github.com/dajiaji/pyseto/actions/workflows/python-package.yml/badge.svg)\n[![codecov](https://codecov.io/gh/dajiaji/pyseto/branch/main/graph/badge.svg?token=QN8GXEYEP3)](https://codecov.io/gh/dajiaji/pyseto)\n\n\nPySETO is a [PASETO (Platform-Agnostic SEcurity TOkens)](https://paseto.io/)/[PASERK (Platform-Agnostic Serialized Keys)](https://github.com/paseto-standard/paserk) implementation written in Python\nwhich supports all of the versions ([v1](https://github.com/paseto-standard/paseto-spec/blob/master/docs/01-Protocol-Versions/Version1.md),\n[v2](https://github.com/paseto-standard/paseto-spec/blob/master/docs/01-Protocol-Versions/Version2.md),\n[v3](https://github.com/paseto-standard/paseto-spec/blob/master/docs/01-Protocol-Versions/Version3.md) and\n[v4](https://github.com/paseto-standard/paseto-spec/blob/master/docs/01-Protocol-Versions/Version4.md)) and purposes (`public` and `local`)\nand has passed all of [the official tests](https://github.com/paseto-standard/test-vectors).\n\nYou can install PySETO with pip:\n\n```sh\n$ pip install pyseto\n```\n\nPySETO can be used in ease as follows (in case of `v4.public`):\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nprivate_key_pem = b"-----BEGIN PRIVATE KEY-----\\nMC4CAQAwBQYDK2VwBCIEILTL+0PfTOIQcn2VPkpxMwf6Gbt9n4UEFDjZ4RuUKjd0\\n-----END PRIVATE KEY-----"\npublic_key_pem = b"-----BEGIN PUBLIC KEY-----\\nMCowBQYDK2VwAyEAHrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI=\\n-----END PUBLIC KEY-----"\n\n\n# Create a PASETO token.\nprivate_key = Key.new(version=4, purpose="public", key=private_key_pem)\ntoken = pyseto.encode(private_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\n# Decode and verify a PASETO token.\npublic_key = Key.new(version=4, purpose="public", key=public_key_pem)\ndecoded = pyseto.decode(public_key, token)\n\nassert token == b\'v4.public.eyJkYXRhIjogInRoaXMgaXMgYSBzaWduZWQgbWVzc2FnZSIsICJleHAiOiAiMjAyMi0wMS0wMVQwMDowMDowMCswMDowMCJ9l1YiKei2FESvHBSGPkn70eFO1hv3tXH0jph1IfZyEfgm3t1DjkYqD5r4aHWZm1eZs_3_bZ9pBQlZGp0DPSdzDg\'\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\nSee following contents or [Documentation](https://pyseto.readthedocs.io/en/stable/) for details.\n\n## Index\n\n- [Installation](#installation)\n- [Supported PASETO Versions](#supported-paseto-versions)\n- [Supported PASERK Types](#supported-paserk-types)\n- [PASETO Usage](#paseto-usage)\n    - [Basic usage: v4.public](#basic-usage-v4public)\n    - [Basic usage: v4.local](#basic-usage-v4local)\n    - [Using serializer/deserializer for payload and footer](#using-serializerdeserializer-for-payload-and-footer)\n    - [Using Paseto class for handling registered claims](#using-paseto-class-for-handling-registered-claims)\n- [PASERK Usage](#paserk-usage)\n    - [Serializing/Deserializing PASERK](#serializingdeserializing-paserk)\n    - [Serializing PASERK ID](#serializing-paserk-id)\n    - [Key Wrapping](#key-wrapping)\n    - [Password-based Key Encryption](#password-based-key-encryption)\n    - [Asymmetric Encryption](#asymmetric-encryption)\n- [API Reference](#api-reference)\n- [Tests](#tests)\n- [Contributing](#contributing)\n\n## Installation\n\nYou can install PySETO with pip:\n\n```sh\n$ pip install pyseto\n```\n\n## Supported PASETO Versions\n\nPySETO supports all of PASETO versions and purposes below:\n\n\n|          |  v4  |  v3  |  v2  |  v1  |\n| ---------| ---- | ---- | ---- | ---- |\n| `local`  |  ✅  |  ✅  |  ✅  |  ✅  |\n| `public` |  ✅  |  ✅  |  ✅  |  ✅  |\n\n\n## Supported PASERK Types\n\nPySETO also supports [PASERK (Platform-Agnostic Serialized Keys)](https://github.com/paseto-standard/paserk).\nCurrently, following PASERK types are supported:\n\n\n|               |  v4  |  v3  |  v2  |  v1  |\n| ------------- | ---- | ---- | ---- | ---- |\n| `lid`         |  ✅  |  ✅  |  ✅  |  ✅  |\n| `sid`         |  ✅  |  ✅  |  ✅  |  ✅  |\n| `pid`         |  ✅  |  ✅  |  ✅  |  ✅  |\n| `local`       |  ✅  |  ✅  |  ✅  |  ✅  |\n| `secret`      |  ✅  |  ✅  |  ✅  |  ✅  |\n| `public`      |  ✅  |  ✅  |  ✅  |  ✅  |\n| `seal`        |  ✅  |      |  ✅  |      |\n| `local-wrap`  |  ✅  |  ✅  |  ✅  |  ✅  |\n| `secret-wrap` |  ✅  |  ✅  |  ✅  |  ✅  |\n| `local-pw`    |  ✅  |  ✅  |  ✅  |  ✅  |\n| `secret-pw`   |  ✅  |  ✅  |  ✅  |  ✅  |\n\n\n## PASETO Usage\n\nBy using this PySETO, you can easily create, decode and verify PASETO tokens. Here are sample codes that handle version 4 PySETO tokens.\n\nPlease refer to [the Documentation](https://pyseto.readthedocs.io/en/stable/) for all usage examples including other versions.\n\n### Basic usage: v4.public\n\n`v4.public` is one of current PASETO versions to be used for asymmetric authentication (public key signatures).\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nprivate_key_pem = b"-----BEGIN PRIVATE KEY-----\\nMC4CAQAwBQYDK2VwBCIEILTL+0PfTOIQcn2VPkpxMwf6Gbt9n4UEFDjZ4RuUKjd0\\n-----END PRIVATE KEY-----"\npublic_key_pem = b"-----BEGIN PUBLIC KEY-----\\nMCowBQYDK2VwAyEAHrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI=\\n-----END PUBLIC KEY-----"\n\nprivate_key = Key.new(version=4, purpose="public", key=private_key_pem)\ntoken = pyseto.encode(private_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\npublic_key = Key.new(version=4, purpose="public", key=public_key_pem)\ndecoded = pyseto.decode(public_key, token)\n\nassert token == b\'v4.public.eyJkYXRhIjogInRoaXMgaXMgYSBzaWduZWQgbWVzc2FnZSIsICJleHAiOiAiMjAyMi0wMS0wMVQwMDowMDowMCswMDowMCJ9l1YiKei2FESvHBSGPkn70eFO1hv3tXH0jph1IfZyEfgm3t1DjkYqD5r4aHWZm1eZs_3_bZ9pBQlZGp0DPSdzDg\'\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\n### Basic usage: v4.local\n\n`v4.local` is one of current PASETO versions to be used for symmetric authenticated encryption.\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nkey = Key.new(version=4, purpose="local", key=b"our-secret")\ntoken = pyseto.encode(key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\ndecoded = pyseto.decode(key, token)\n\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\n### Using serializer/deserializer for payload and footer\n\nBy using `serializer` and `deserializer`, you can encode/decode a dict-typed payload and footer included in PASETO tokens into an arbitrary format.\nThe following example shows that the payload and the footer in a PASETO token are encoded/decoded as JSON formatted data.\nWhen specifing dict-typed payload, exp parameter can be used to set the expiration time (seconds) of the token.\n\n```py\nimport json\nimport pyseto\nfrom pyseto import Key\n\nprivate_key_pem = b"-----BEGIN PRIVATE KEY-----\\nMC4CAQAwBQYDK2VwBCIEILTL+0PfTOIQcn2VPkpxMwf6Gbt9n4UEFDjZ4RuUKjd0\\n-----END PRIVATE KEY-----"\npublic_key_pem = b"-----BEGIN PUBLIC KEY-----\\nMCowBQYDK2VwAyEAHrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI=\\n-----END PUBLIC KEY-----"\n\nprivate_key = Key.new(version=4, purpose="public", key=private_key_pem)\npublic_key = Key.new(version=4, purpose="public", key=public_key_pem)\ntoken = pyseto.encode(\n    private_key,\n    {"data": "this is a signed message"},\n    footer={"kid": public_key.to_paserk_id()},\n    serializer=json,\n    exp=3600,\n)\n\ndecoded = pyseto.decode(public_key, token, deserializer=json)\n\nassert decoded.payload["data"] == "this is a signed message"\nassert decoded.payload["exp"] == "2021-11-11T00:00:00+00:00"\nassert decoded.footer["kid"] == "k4.pid.yh4-bJYjOYAG6CWy0zsfPmpKylxS7uAWrxqVmBN2KAiJ"\n```\n\n### Using `Paseto` class for handling registered claims\n\nBy using `Paseto` class, you can change the default value of `exp` (the expiration date ot tokens), whether to include an `iat` claim, and other settings.\n\nNote that `pyseto.encode()` and `pyseto.decode()` are aliases to the `encode()` and `decode()` of the global "Paseto" class instance created with the default settings.\n\n```py\nimport json\nimport pyseto\nfrom pyseto import Key, Paseto\n\nprivate_key_pem = b"-----BEGIN PRIVATE KEY-----\\nMC4CAQAwBQYDK2VwBCIEILTL+0PfTOIQcn2VPkpxMwf6Gbt9n4UEFDjZ4RuUKjd0\\n-----END PRIVATE KEY-----"\npublic_key_pem = b"-----BEGIN PUBLIC KEY-----\\nMCowBQYDK2VwAyEAHrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI=\\n-----END PUBLIC KEY-----"\n\nprivate_key = Key.new(version=4, purpose="public", key=private_key_pem)\npaseto = Paseto.new(exp=3600, include_iat=True) # Default values are exp=0(not specified) and including_iat=False\ntoken = paseto.encode(\n    private_key,\n    {"data": "this is a signed message"},\n    serializer=json,\n)\n\npublic_key = Key.new(version=4, purpose="public", key=public_key_pem)\ndecoded = pyseto.decode(public_key, token, deserializer=json)\n\nassert decoded.payload["data"] == "this is a signed message"\nassert decoded.payload["iat"] == "2021-11-11T00:00:00+00:00"\nassert decoded.payload["exp"] == "2021-11-11T01:00:00+00:00"\n```\n\n## PASERK Usage\n\n[PASERK (Platform-Agnostic Serialized Keys)](https://github.com/paseto-standard/paserk) is an extension to PASETO that provides key-wrapping and serialization.\n\n### Serializing/Deserializing PASERK\n\nAs shown in the examples above, the `pyseto.Key` used for encryption and signature can be generated from PASERK or converted to PASERK as follows:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\n# pyseto.Key can be generated from PASERK.\nsymmetric_key = Key.new(version=4, purpose="local", key=b"our-secret")\nprivate_key = Key.from_paserk("k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog")\npublic_key = Key.from_paserk("k4.public.Hrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI")\n\ntoken = pyseto.encode(private_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\ndecoded = pyseto.decode(public_key, token)\n\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n\n# PASERK can be derived from pyseto.Key.\nassert symmetric_key.to_paserk() == "k4.local.b3VyLXNlY3JldA"\nassert private_key.to_paserk() == "k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog"\nassert public_key.to_paserk() == "k4.public.Hrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI"\n```\n\n### Serializing PASERK ID\n\n`pyseto.Key` can also be converted to PASERK ID as follows:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\n# pyseto.Key can be generated from PASERK.\nsymmetric_key = Key.new(version=4, purpose="local", key=b"our-secret")\nprivate_key = Key.from_paserk("k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog")\npublic_key = Key.from_paserk("k4.public.Hrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI")\n\n# PASERK ID can be derived from pyseto.Key.\nassert symmetric_key.to_paserk_id() == "k4.lid._D6kgTzxgiPGk35gMj9bukgj4En2H94u22wVX9zaoh05"\nassert private_key.to_paserk() == "k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog"\nassert public_key.to_paserk_id() == "k4.pid.yh4-bJYjOYAG6CWy0zsfPmpKylxS7uAWrxqVmBN2KAiJ"\n```\n\n### Key Wrapping\n\nIf you call `to_paserk` with `wrapping_key`, you can get a wrapped (encrypted) PASERK with the wrapping key.\nThe wrapped PASERK can be decrypted by calling `from_paserk` with `wrapping key`.\n\nIn case of `local-wrap.pie`:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nraw_key = Key.new(version=4, purpose="local", key=b"our-secret")\nwrapping_key = token_bytes(32)\nwpk = raw_key.to_paserk(wrapping_key=wrapping_key)\ntoken = pyseto.encode(raw_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\nunwrapped_key = Key.from_paserk(wpk, wrapping_key=wrapping_key)\ndecoded = pyseto.decode(unwrapped_key, token)\n\n# assert wpk == "k4.local-wrap.pie.TNKEwC4K1xBcgJ_GiwWAoRlQFE33HJO3oN9DHEZ05pieSCd-W7bgAL64VG9TZ_pBkuNBFHNrfOGHtnfnhYGdbz5-x3CxShhPJxg"\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\nIn case of `secret-wrap.pie`:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nraw_private_key = Key.from_paserk(\n    "k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog"\n)\nwrapping_key = token_bytes(32)\nwpk = raw_private_key.to_paserk(wrapping_key=wrapping_key)\nunwrapped_private_key = Key.from_paserk(wpk, wrapping_key=wrapping_key)\ntoken = pyseto.encode(unwrapped_private_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\npublic_key = Key.from_paserk("k4.public.Hrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI")\ndecoded = pyseto.decode(public_key, token)\n\n# assert wpk == "k4.secret-wrap.pie.excv7V4-NaECy5hpji-tkSkMvyjsAgNxA-mGALgdjyvGNyDlTb89bJ35R1e3tILgbMpEW5WXMXzySe2T-sBz-ZAcs1j7rbD3ZWvsBTM6K5N9wWfAxbR4ppCXH_H5__9yY-kBaF2NimyAJyduhOhSmqLm6TTSucpAOakEJOXePW8"\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\n### Password-based Key Encryption\n\nIf you call `to_paserk` with `password`, you can get a wrapped (encrypted) PASERK with the password.\nThe wrapped PASERK can be decrypted by calling `from_paserk` with `passwrod`.\n\nIn case of `local-pw`:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nraw_key = Key.new(version=4, purpose="local", key=b"our-secret")\ntoken = pyseto.encode(raw_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\nwpk = raw_key.to_paserk(password="our-secret")\nunwrapped_key = Key.from_paserk(wpk, password="our-secret")\ndecoded = pyseto.decode(unwrapped_key, token)\n\n# assert wpk == "k4.local-pw.HrCs9Pu-2LB0l7jkHB-x2gAAAAAA8AAAAAAAAgAAAAGttW0IHZjQCHJdg-Vc3tqO_GSLR4vzLl-yrKk2I-l8YHj6jWpC0lQB2Z7uzTtVyV1rd_EZQPzHdw5VOtyucP0FkCU"\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\nIn case of `secret-pw`:\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nraw_private_key = Key.from_paserk(\n    "k4.secret.tMv7Q99M4hByfZU-SnEzB_oZu32fhQQUONnhG5QqN3Qeudu7vAR8A_1wYE4AcfCYfhayi3VyJcEfAEFdDiCxog"\n)\nwpk = raw_private_key.to_paserk(password="our-secret")\nunwrapped_private_key = Key.from_paserk(wpk, password="our-secret")\ntoken = pyseto.encode(unwrapped_private_key, b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\')\n\npublic_key = Key.from_paserk(\n    "k4.public.Hrnbu7wEfAP9cGBOAHHwmH4Wsot1ciXBHwBBXQ4gsaI"\n)\ndecoded = pyseto.decode(public_key, token)\n\n# assert wpk == "k4.secret-pw.MEMW4K1MaD5nWigCLyEyFAAAAAAA8AAAAAAAAgAAAAFU-tArtryNVjS2n2hCYiM11V6tOyuIog69Bjb0yNZanrLJ3afGclb3kPzQ6IhK8ob9E4QgRdEALGWCizZ0RCPFF_M95IQDfmdYKC0Er656UgKUK4UKG9JlxP4o81UwoJoZYz_D1zTlltipEa5RiNvUtNU8vLKoGSY"\nassert decoded.payload == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n```\n\n### Asymmetric Encryption\n\nAt this time, PySETO supports asymmetric encryption (key sealing) for `v2` and `v4`.\n\n```py\nimport pyseto\nfrom pyseto import Key\n\nprivate_key_pem = b"-----BEGIN PRIVATE KEY-----\\nMC4CAQAwBQYDK2VuBCIEIFAF7jSCZHFgWvC8hUkXr55Az6Pot2g4zOAUxck0/6x8\\n-----END PRIVATE KEY-----"\npublic_key_pem = b"-----BEGIN PUBLIC KEY-----\\nMCowBQYDK2VuAyEAFv8IXsICYj0paznDK/99GyCsFOIGnfY87ayyNSIvSB4=\\n-----END PUBLIC KEY-----"\n\nraw_key = Key.new(version=4, purpose="local", key=b"our-secret")\ntoken = pyseto.encode(\n    raw_key,\n    b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\',\n)\n\nsealed_key = raw_key.to_paserk(sealing_key=public_key_pem)\nunsealed_key = Key.from_paserk(sealed_key, unsealing_key=private_key_pem)\ndecoded = pyseto.decode(unsealed_key, token)\n\nassert (\n    decoded.payload\n    == b\'{"data": "this is a signed message", "exp": "2022-01-01T00:00:00+00:00"}\'\n)\n```\n\nKey searing for `v1` and `v3` have not been supported yet.\n\n\n## API Reference\n\nSee [Documentation](https://pyseto.readthedocs.io/en/stable/api.html).\n\n## Tests\n\nYou can run tests from the project root after cloning with:\n\n```sh\n$ tox\n```\n\n## Contributing\n\nWe welcome all kind of contributions, filing issues, suggesting new features or sending PRs.\n',
    'author': 'Ajitomi Daisuke',
    'author_email': 'dajiaji@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dajiaji/pyseto',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



from typing import Any, Union

from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric.ed25519 import (
    Ed25519PrivateKey,
    Ed25519PublicKey,
)
from cryptography.hazmat.primitives.serialization import (
    load_pem_private_key,
    load_pem_public_key,
)

from .key_interface import KeyInterface
from .versions.v1 import V1Local, V1Public
from .versions.v2 import V2Local, V2Public
from .versions.v3 import V3Local, V3Public
from .versions.v4 import V4Local, V4Public


class Key:
    """
    Tha factory methods for PASETO keys.
    """

    _PASERK_TYPE_SUPPORTED = [
        "local",
        "public",
        "secret",
        "seal",
        "local-wrap",
        "secret-wrap",
        "local-pw",
        "secret-pw",
    ]

    @classmethod
    def new(cls, version: int, purpose: str, key: Union[bytes, str] = b"") -> KeyInterface:

        """
        Constructor of a PASETO key object which has
        :class:`KeyInterface <pyseto.key_interface.KeyInterface>`.

        Args:
            version(int): The version of the key. It will be ``1``,
                ``2``, ``3`` or ``4``.
            purpose (str): The purpose of the key. It will be ``public`` or
                ``local``.
            key (Union[bytes, str]): A key itself or keying material.
        Returns:
            KeyInterface: A PASETO key object.
        Raise:
            ValueError: Invalid arguments.
        """
        bkey = key if isinstance(key, bytes) else key.encode("utf-8")
        if purpose == "local":
            return cls._create_private_key(version, bkey)

        elif purpose == "public":
            k: Any = None
            if bkey.startswith(b"-----BEGIN EC PRIVATE"):
                k = load_pem_private_key(bkey, password=None)
            elif bkey.startswith(b"-----BEGIN PRIVATE"):
                k = load_pem_private_key(bkey, password=None)
            elif bkey.startswith(b"-----BEGIN PUBLIC"):
                k = load_pem_public_key(bkey)
            elif bkey.startswith(b"-----BEGIN RSA PRIVATE"):
                k = load_pem_private_key(bkey, password=None)
            else:
                raise ValueError("Invalid or unsupported PEM format.")
            return cls._create_public_key(version, k)

        raise ValueError(f"Invalid purpose: {purpose}.")

    @classmethod
    def from_paserk(
        cls,
        paserk: str,
        wrapping_key: Union[bytes, str] = b"",
        password: Union[bytes, str] = b"",
        unsealing_key: Union[bytes, str] = b"",
    ) -> KeyInterface:

        """
        Generates a PASETO key object which has
        :class:`KeyInterface <pyseto.key_interface.KeyInterface>` from PASERK.

        Args:
            paserk (str): A PASERK string.
            wrapping_key (Union[bytes, str]): A wrapping key. If the
                `wrapping_key` is specified, `password` should not be specified.
            password (Union[bytes, str]): A password for key wrapping. If the
                `password` is specified, `wrapping_key` should not be specified.
            unsealing_key (Union[bytes, str]): A password for key wrapping. If the
                `password` is specified, `wrapping_key` should not be specified.
        Returns:
            KeyInterface: A PASETO key object.
        Raise:
            ValueError: Invalid arguments.
        """

        frags = paserk.split(".")
        if frags[1] not in cls._PASERK_TYPE_SUPPORTED:
            raise ValueError(f"Invalid PASERK key type: {frags[1]}.")
        bwk = wrapping_key if isinstance(wrapping_key, bytes) else wrapping_key.encode("utf-8")
        bpw = password if isinstance(password, bytes) else password.encode("utf-8")
        bsk = unsealing_key if isinstance(unsealing_key, bytes) else unsealing_key.encode("utf-8")

        if frags[0] == "k1":
            return (
                V1Local.from_paserk(paserk, bwk, bpw, bsk)
                if frags[1].startswith("local") or frags[1] == "seal"
                else V1Public.from_paserk(paserk, bwk, bpw)
            )
        if frags[0] == "k2":
            return (
                V2Local.from_paserk(paserk, bwk, bpw, bsk)
                if frags[1].startswith("local") or frags[1] == "seal"
                else V2Public.from_paserk(paserk, bwk, bpw)
            )
        if frags[0] == "k3":
            return (
                V3Local.from_paserk(paserk, bwk, bpw, bsk)
                if frags[1].startswith("local") or frags[1] == "seal"
                else V3Public.from_paserk(paserk, bwk, bpw)
            )
        if frags[0] == "k4":
            return (
                V4Local.from_paserk(paserk, bwk, bpw, bsk)
                if frags[1].startswith("local") or frags[1] == "seal"
                else V4Public.from_paserk(paserk, bwk, bpw)
            )
        raise ValueError(f"Invalid PASERK version: {frags[0]}.")

    # @classmethod
    # def from_public_bytes(cls, version: int, key: bytes) -> KeyInterface:
    #     if version == 1:
    #         raise ValueError(f"RSA key is not supported.")
    #     if version == 2:
    #         return V2Public.from_public_bytes(key)
    #     if version == 3:
    #         return V3Public.from_public_bytes(key)
    #     if version == 4:
    #         return V4Public.from_public_bytes(key)
    #     raise ValueError(f"Invalid version: {version}.")

    @staticmethod
    def from_asymmetric_key_params(version: int, x: bytes = b"", y: bytes = b"", d: bytes = b"") -> KeyInterface:

        """
        Constructor of a PASETO key object which has
        :class:`KeyInterface <pyseto.key_interface.KeyInterface>` wth
        asymmetric key parameters (x-coordinate, y-coordinate, and/or private
        key). This is intended to be used to generate keys for PASETO from JWK
        and other sources.

        Args:
            version(int): The version of the key. It will be ``1``,
                ``2``, ``3`` or ``4``.
            x (bytes): The x coordinate of the key.
            y (bytes): The y coordinate of the key.
            d (bytes): The private key component of the key.
        Returns:
            KeyInterface: A PASETO key object.
        Raise:
            ValueError: Invalid arguments.
        """
        k: Any = None

        if version == 1:
            raise ValueError("v1.public is not supported on from_key_parameters.")

        if version == 2:
            if x and d:
                raise ValueError("Only one of x or d should be set for v2.public.")
            if x:
                try:
                    k = Ed25519PublicKey.from_public_bytes(x)
                except Exception as err:
                    raise ValueError("Failed to load key.") from err
                return V2Public(k)
            if d:
                try:
                    k = Ed25519PrivateKey.from_private_bytes(d)
                except Exception as err:
                    raise ValueError("Failed to load key.") from err
                return V2Public(k)
            raise ValueError("x or d should be set for v2.public.")

        if version == 3:
            if not x or not y:
                raise ValueError("x and y (and d) should be set for v3.public.")
            try:
                pn = ec.EllipticCurvePublicNumbers(
                    x=int.from_bytes(x, byteorder="big"),
                    y=int.from_bytes(y, byteorder="big"),
                    curve=ec.SECP384R1(),
                )
                k = pn.public_key()
            except Exception as err:
                raise ValueError("Failed to load key.") from err

            if not d:
                return V3Public(k)
            try:
                k = ec.EllipticCurvePrivateNumbers(int.from_bytes(d, byteorder="big"), pn).private_key()
            except Exception as err:
                raise ValueError("Failed to load key.") from err
            return V3Public(k)

        if version == 4:
            if x and d:
                raise ValueError("Only one of x or d should be set for v4.public.")
            if x:
                try:
                    k = Ed25519PublicKey.from_public_bytes(x)
                except Exception as err:
                    raise ValueError("Failed to load key.") from err
                return V4Public(k)
            if d:
                try:
                    k = Ed25519PrivateKey.from_private_bytes(d)
                except Exception as err:
                    raise ValueError("Failed to load key.") from err
                return V4Public(k)
            raise ValueError("x or d should be set for v4.public.")

        raise ValueError(f"Invalid version: {version}.")

    @staticmethod
    def _create_public_key(version: int, key: Any) -> KeyInterface:
        if version == 1:
            return V1Public(key)
        if version == 2:
            return V2Public(key)
        if version == 3:
            return V3Public(key)
        if version == 4:
            return V4Public(key)
        raise ValueError(f"Invalid version: {version}.")

    @staticmethod
    def _create_private_key(version: int, key: bytes) -> KeyInterface:
        if version == 1:
            return V1Local(key)
        if version == 2:
            return V2Local(key)
        if version == 3:
            return V3Local(key)
        if version == 4:
            return V4Local(key)
        raise ValueError(f"Invalid version: {version}.")
